
#include <QBoxLayout>
#include <QAction>
#include <QFileInfo>
#include <QPushButton>

#include <math.h>
#include "ImageSelectionList.h"
#include "ImageSearchThread.h"
#include "wpscrollarea.h"
#include "CheckableListItem.h"


ImageSelectionList::ImageSelectionList(const QStringList& aInitialSelections, QWidget* aParent)
    : MDragWidget(aParent), iInitialSelections(aInitialSelections)
{
#ifdef Q_WS_MAEMO_5
	setWindowFlags(Qt::Window);
	setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
	setWindowTitle(tr("Image selection"));
	
	QVBoxLayout* layout = new QVBoxLayout(this);
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);
	iScrollArea = new WPScrollArea(this);
	iScrollArea->setFocusPolicy(Qt::NoFocus);
	layout->addWidget(iScrollArea);
	
#ifdef Q_OS_SYMBIAN
	QAction* leftSoftKey = new QAction(tr("Select"), this);
	connect(leftSoftKey, SIGNAL(triggered()), this, SLOT(ItemsSelected()));
	leftSoftKey->setSoftKeyRole(QAction::PositiveSoftKey);
	addAction(leftSoftKey);
	QAction* rightSoftKey = new QAction(tr("Cancel"), this);
	connect(rightSoftKey, SIGNAL(triggered()), this, SLOT(CancelClicked()));
	rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
	addAction(rightSoftKey);
#else
	QPushButton* selectButton = new QPushButton(tr("Select"));
	selectButton->setObjectName("Maemo5NavigationButton");
	connect(selectButton, SIGNAL(clicked()), this, SLOT(ItemsSelected()));
	QHBoxLayout* buttonLayout = new QHBoxLayout;
	buttonLayout->addWidget(selectButton);
#ifndef Q_WS_MAEMO_5
	QPushButton* cancelButton = new QPushButton(tr("Cancel"));
	cancelButton->setObjectName("Maemo5NavigationButton");
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(CancelClicked()));
	buttonLayout->addWidget(cancelButton);
#endif
	layout->addLayout(buttonLayout);
#endif
	setLayout(layout);
}

ImageSelectionList::~ImageSelectionList()
{
	if (iSearchThread)
	{
		if (iSearchThread->isRunning())
		{
			iSearchThread->Cancel();
			iSearchThread->wait();
		}
		delete iSearchThread;
	}
}

void ImageSelectionList::SearchImages()
{
	emit TaskStarted(EIdle);
	iSearchThread = new ImageSearchThread(this);
	qRegisterMetaType<QFileInfo>("QFileInfo");	
	connect(iSearchThread, SIGNAL(ImageFound(const QFileInfo&)), this, SLOT(ImageFound(const QFileInfo&)));
	connect(iSearchThread, SIGNAL(finished()), this, SLOT(ThreadFinished()));
	iSearchThread->start();
}

void ImageSelectionList::ImageFound(const QFileInfo& aFileInfo)
{
	iFiles.append(new QFileInfo(aFileInfo));
}

void ImageSelectionList::CancelClicked()
{
	emit ImagesSelected(iInitialSelections);
}

void ImageSelectionList::ItemsSelected()
{
	QStringList filenames;
	const int count = iItems.count();
	for (int i = 0; i < count; i++)
	{
		if (iItems[i]->IsChecked())
			filenames.append(iItems[i]->ItemId());
	}
	emit ImagesSelected(filenames);
#ifdef Q_WS_MAEMO_5
	close();
#endif
}

void ImageSelectionList::ThreadFinished()
{
	if (!iScrollArea->widget())
		iScrollArea->InitLayout();
	iScrollArea->widget()->setMinimumHeight(iFiles.count() * 80);
	foreach (const QFileInfo* aFileInfo, iFiles)
	{
		CheckableListItem* item = new CheckableListItem(CheckableListItem::EImage,
														aFileInfo->filePath(),
														iInitialSelections.contains(aFileInfo->filePath()),
														*this);
		item->setFixedHeight(80);
		item->InitImageItem(aFileInfo->fileName(),
							WPNetworkEngine::LocalizedTime(aFileInfo->lastModified()),
							FormatFileSize(aFileInfo->size()));
		
		iScrollArea->addWidget(item);
		iItems.append(item);
		// Used, so remember to delete
		delete aFileInfo;
	}
	
	emit TaskFinished(EIdle);
	
	iFiles.clear();
	delete iSearchThread;
	iSearchThread = NULL;
	iScrollArea->AddStretch();
}

bool ImageSelectionList::MDragWidgetIsDragging() const
{
	return iScrollArea->IsDragging();
}

QSize ImageSelectionList::VisibleSize() const
{
	return iScrollArea->GetVisibleContentSize();
}

QString ImageSelectionList::FormatFileSize(int aFileSize) const
{
	// TODO make something more fancy
	return QString::number(floor(aFileSize / 1024)) + tr(" kB");
}

void ImageSelectionList::setFocus()
{
	if (iItems.count() > 0)
		iItems.at(0)->setFocus();
	else
		iScrollArea->setFocus();
}
