/*
 * PostsView.cpp
 *
 *  Created on: 4.11.2009
 *      Author: thyv
 */

#include <QDateTime>
#include <QVBoxLayout>
#include <QDesktopWidget>

#include "PostsView.h"
#include "wpscrollarea.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"

PostsView::PostsView(BlogView *aParent) : QWidget(aParent), iParent(aParent)
{
	QVBoxLayout* layout = new QVBoxLayout(this);
	iScrollArea = new WPScrollArea;
	iScrollArea->setFocusPolicy(Qt::NoFocus);
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(iScrollArea);

	iLocalDraftList = new CollectionList(this, iScrollArea);
	iLocalDraftList->SetTitle(tr("Drafted posts "), true);

	connect(iLocalDraftList->iActionOpen, SIGNAL(triggered()), this, SLOT(OpenLocalDraft()));
	connect(iLocalDraftList->iActionEdit, SIGNAL(triggered()), this, SLOT(EditLocalDraft()));
	connect(iLocalDraftList->iActionDelete, SIGNAL(triggered()), this, SLOT(DeleteLocalDraft()));
	
	iPostList = new CollectionList(this, iScrollArea);
	iPostList->SetTitle(tr("Published posts "));
	
	connect(iPostList->iActionOpen, SIGNAL(triggered()), this, SLOT(Open()));
	connect(iPostList->iActionEdit, SIGNAL(triggered()), this, SLOT(Edit()));
	connect(iPostList->iActionDelete, SIGNAL(triggered()), this, SLOT(Delete()));
	
	iScrollArea->addWidget(iLocalDraftList);
	iScrollArea->addWidget(iPostList);
}

PostsView::~PostsView()
{
}

void PostsView::AddPost(const WPDataPost& aPost, int aCommentCount)
{	
	QStringList list;
	if (aPost.IsLocalDraft())
		list << "Local draft";
	list << WPNetworkEngine::LocalizedTime(aPost.iDateCreated);
	
	if (aCommentCount > 0)
	{
		QString comments(QString::number(aCommentCount));
		comments += aCommentCount > 1 ? tr(" comments") : tr(" comment");
		list << comments;
	}
	else
	{
		list << tr("No comments");
	}
	if (aPost.IsDraftOrPending())
		iLocalDraftList->AddCustomItem(aPost.iPostId, aPost.iTitle, aPost.iWpAuthorDisplayName, list);
	else
		iPostList->AddCustomItem(aPost.iPostId, aPost.iTitle, aPost.iWpAuthorDisplayName, list);
}

void PostsView::UpdatePostData(const WPDataBlog& aBlog)
{
	QList<WPDataPost> draftPosts;
	QList<WPDataPost> publishedPosts;
	
	iLocalDraftList->ResetList();
	iPostList->ResetList();
	
	const int localDraftCount = aBlog.iLocalDraftPosts.count();
	const int postCount = aBlog.iPosts.count();
	const int commentCount = aBlog.iComments.count();
	
	for (int i = 0; i < localDraftCount; ++i)
	{
		// Local drafts can't have comments
		AddPost(aBlog.iLocalDraftPosts[i], 0);
	}
	for (int i = 0; i < postCount; ++i)
	{
		if (aBlog.iPosts[i].IsDraftOrPending())
			draftPosts.append(aBlog.iPosts[i]);
		else
			publishedPosts.append(aBlog.iPosts[i]);
	}
	
	const int draftCount = draftPosts.count();
	const int publishedCount = publishedPosts.count();
	
	for (int i = 0; i < draftCount; ++i)
	{
		int comments = 0;
		for (int j = 0; j < commentCount; ++j)
			{
			if (aBlog.iComments[j].iPostId.compare(draftPosts[i].iPostId) == 0)
				++comments;
			}
		
		AddPost(draftPosts[i], comments);
	}
	for (int i = 0; i < publishedCount; ++i)
	{
		int comments = 0;
		for (int j = 0; j < commentCount; ++j)
			{
			if (aBlog.iComments[j].iPostId.compare(publishedPosts[i].iPostId) == 0)
				++comments;
			}
		
		AddPost(publishedPosts[i], comments);
	}
	iLocalDraftList->SetTitle();
	iPostList->SetTitle();
	iScrollArea->AddStretch();
}

void PostsView::UpdateCommentsData(const WPDataBlog& aBlog)	
{
	//TODO: implement update functionality.
	UpdatePostData(aBlog);
}

void PostsView::Open()
{
	emit OpenClicked(iPostList->iCurrentItem);
}

void PostsView::Edit()
{
	emit EditClicked(iPostList->iCurrentItem);
}

void PostsView::Delete()
{
	emit DeleteClicked(iPostList->iCurrentItem);
}

void PostsView::OpenLocalDraft()
{
	emit OpenClicked(iLocalDraftList->iCurrentItem);
}

void PostsView::EditLocalDraft()
{
	emit EditClicked(iLocalDraftList->iCurrentItem);
}

void PostsView::DeleteLocalDraft()
{
	if (WPDataPost::IsLocalDraftId(iLocalDraftList->iCurrentItem->ItemId()))
	{
		// Local draft -> delete locally
		WPDataBlog* blog = iParent->GetNetworkEngine()->GetCurrentBlog();
		blog->RemoveLocalDraftPost(iLocalDraftList->iCurrentItem->ItemId());
		iLocalDraftList->RemoveItem(iLocalDraftList->iCurrentItem);
		setFocus();
	}
	else
	{
		// Network draft -> delete as a normal post
		emit DeleteClicked(iLocalDraftList->iCurrentItem);
	}
}

void PostsView::setFocus()
{
	if (iLocalDraftList && iLocalDraftList->ItemCount() > 0)
		iLocalDraftList->setFocus();
	else if (iPostList && iPostList->ItemCount() > 0)
	    iPostList->setFocus();
	else
	    QWidget::setFocus();
}
