/*
 * WPNetworkEngine.cpp
 *
 *  Created on: 4.11.2009
 *      Author: jkal
 */

#include <QtGlobal>
#include <QBuffer>
#include <QDebug>
#include <QNetworkReply>
#include <QStringList>
#include <QTreeWidget>
#include <QFile>
#include <QDataStream>
#include <QListWidgetItem>
#include <QImageReader>
#include <QMessageBox>
#include <QDir>
#include <QDesktopServices>

#include "WPNetworkEngine.h"
#include "WPXmlUsersBlogsParser.h"
#include "WPXmlPagesParser.h"
#include "WPXmlPostsParser.h"
#include "WPXmlCategoriesParser.h"
#include "WPXmlTagsParser.h"
#include "WPXmlStatusesParser.h"
#include "WPXmlCommentsParser.h"
#include "WPXmlRsdParser.h"
#include "WPXmlResponseParser.h"
#include "WPXmlFileUploadParser.h"
#include "WPXmlStatsParser.h"
#include "WPData.h"
#include "md5.h"
#include "WPLogger.h"

const char* KDefaultAvatar = ":/images/default-avatar.png";

WPNetworkQueueItem::WPNetworkQueueItem()
: iNetworkReply(NULL), iShowWaitNote(false), iShowErrorNote(false)
{
}

WPNetworkQueueItem::WPNetworkQueueItem(QNetworkReply* aNetworkReply, bool aShowWaitNote, bool aShowErrorNote)
: iNetworkReply(aNetworkReply), iShowWaitNote(aShowWaitNote), iShowErrorNote(aShowErrorNote)
{
}

WPNetworkEngine::WPNetworkEngine()
{
#ifndef Q_OS_SYMBIAN
    QString path(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
    path += ".wordpress/";
    QDir dir(path);
    if (!dir.exists())
        dir.mkpath(path);
#endif

	iDataUsersBlogs = new WPDataUsersBlogs;
	iDataUsersBlogs->iActiveBlogIndex = 0;
	iNetworkAccessManager = new QNetworkAccessManager(this);
	
	connect(iNetworkAccessManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(HandleNetworkData(QNetworkReply*)));
}

WPNetworkEngine::~WPNetworkEngine()
{
	//ClearRequestQueue();
	delete iDataUsersBlogs;
}

void WPNetworkEngine::RefreshAll(bool aShowWaitSign)
{
	// We could update progress bar here
	if (iDataUsersBlogs->iBlogs.size() == 0)
		return;
	// These will emit data update signals
	GetComments(aShowWaitSign);
	GetPosts(aShowWaitSign);
	GetPages(aShowWaitSign);
	GetCategories(aShowWaitSign);
	// These won't
	GetTags();
	GetPageStatuses();
	GetPostStatuses();
	GetCommentStatuses();

	GetStats();
}

void WPNetworkEngine::OpenBlog(const QString& aBlogId)
{
	QString currentBlog = iBlogData->iRpcUrl;
	currentBlog += iBlogData->iBlogId;
	if(currentBlog.compare(aBlogId) == 0) //Same blog was selected so nothing to do.
		return; 

	const int blogCount = iDataUsersBlogs->iBlogs.count();
	for (int i = 0; i < blogCount; ++i)
	{
		WPDataBlog& blog = iDataUsersBlogs->iBlogs[i];
		QString id = blog.iRpcUrl;
		id += blog.iBlogId;
		
		if (id.compare(aBlogId) == 0)
		{
			// First, let's remove all pending requests
			ClearRequestQueue();

			iDataUsersBlogs->iActiveBlogIndex = i;
			SetCurrentBlog(blog);
			emit UpdateData(blog);
			emit UpdateCommentsData(*iBlogData);
			emit UpdatePageData(*iBlogData);
			emit UpdateCategoryData(*iBlogData);

			RefreshAll(true);
			return;
		}
	}
}

WPDataBlog* WPNetworkEngine::GetBlog(const QString& aBlogId)
{
	const int blogCount = iDataUsersBlogs->iBlogs.count();
	for (int i = 0; i < blogCount; ++i)
	{
		WPDataBlog& blog = iDataUsersBlogs->iBlogs[i];
		QString id = blog.iRpcUrl;
		id += blog.iBlogId;
		if (id == aBlogId)
			return &blog;
	}
	return 0;
}

void WPNetworkEngine::SaveAllData()
{
#ifdef Q_OS_SYMBIAN
	QFile file("data.txt");
#else
	QString path(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
    path += ".wordpress/";
	QDir dir(path);
	if (!dir.exists())
		dir.mkpath(path);
	
	path.append("data.txt");
	
	QFile file(path);
#endif
	if(!file.open(QIODevice::WriteOnly))
		return;
	QDataStream out(&file);
	out.setVersion(QDataStream::Qt_4_6);
	out << *iDataUsersBlogs;
}

bool WPNetworkEngine::LoadLocalData()
{
#ifdef Q_OS_SYMBIAN
	QFile file("data.txt");
#else
	QString path(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
	path.append(".wordpress/data.txt");
	QFile file(path);
#endif
	if(!file.open(QIODevice::ReadOnly))
		return false;
	QDataStream in(&file);
	in.setVersion(QDataStream::Qt_4_6);
	in >> *iDataUsersBlogs;
	
	if (iDataUsersBlogs->iBlogs.count())
	{
		const int index = iDataUsersBlogs->iActiveBlogIndex <
				iDataUsersBlogs->iBlogs.count() ? iDataUsersBlogs->iActiveBlogIndex : 0;
		SetCurrentBlog(iDataUsersBlogs->iBlogs[index]);
		emit UpdateData(*iDataUsersBlogs);
		emit UpdateCommentsData(*iBlogData);
		emit UpdatePageData(*iBlogData);
		emit UpdateCategoryData(*iBlogData);
		return true;
	}
	return false;
}

void WPNetworkEngine::SaveBlogSetup(const QString& aBlogUrl, const QString& aUsername,
		const QString& aPassword, const QString& aNumberOfPosts, bool aResizePhotos)
{
    // Let's first check whether the user supplied URL string is valid.
    QUrl url = QUrl::fromUserInput(aBlogUrl);
    if (url == QUrl())
    {
        emit ShowErrorNote(tr("Invalid blog URL."));
        return;
    }

	ClearRequestQueue();

	iUsername = aUsername;
	iPassword = aPassword;
	iNumberOfPosts = aNumberOfPosts;
	iResizePhotos = aResizePhotos;
    
	// We will show an error note if this fails as it is the first error point in network error situations
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->get(CreateNetworkRequest(url, EGetMainPage)), true, true));
	emit TaskStarted(EGetMainPage);
}

WPDataPost* WPNetworkEngine::GetPost(QString aPostId)
{
	WPDataPost* post = 0;
	QMutableListIterator<WPDataPost> i(iBlogData->iPosts);
	while (i.hasNext())
	{
		post = &(i.next());
		if (post->iPostId != aPostId)
			post = 0;
		else
			break;
	}
	return post;
}

WPDataPost* WPNetworkEngine::GetLocalDraftPost(QString aPostId)
{
	WPDataPost* post = 0;
	QMutableListIterator<WPDataPost> i(iBlogData->iLocalDraftPosts);
	while (i.hasNext())
	{
		post = &(i.next());
		if (post->iPostId != aPostId)
			post = 0;
		else
			break;
	}
	return post;
}

WPDataPage* WPNetworkEngine::GetLocalDraftPage(QString aPageId)
{
	WPDataPage* page = 0;
	QMutableListIterator<WPDataPage> i(iBlogData->iLocalDraftPages);
	while (i.hasNext())
	{
		page = &(i.next());
		if (page->iPageId != aPageId)
			page = 0;
		else
			break;
	}
	return page;
}

void WPNetworkEngine::GetUsersBlogs(const QUrl& aUrl)
{
	QStringList params;
	params << "wp.getUsersBlogs" << iUsername << iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(aUrl, EGetUsersBlogs), body.toUtf8()), true, true));
	emit TaskStarted(EGetUsersBlogs);
}

void WPNetworkEngine::GetPages(bool aShowWaitSign)
	{
	QStringList params;
	params << "wp.getPages" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);

	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetPages), body.toUtf8()), aShowWaitSign, false));
	if (aShowWaitSign)
		emit TaskStarted(EGetPages);
	}

void WPNetworkEngine::GetPosts(bool aShowWaitSign)
	{
	QStringList params;
	params << "metaWeblog.getRecentPosts" << iBlogData->iBlogId << iBlogData->iUsername
			<< iBlogData->iPassword << iBlogData->iNumberOfPosts;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetPosts), body.toUtf8()), aShowWaitSign, false));
	if (aShowWaitSign)
		emit TaskStarted(EGetPosts);
	}

void WPNetworkEngine::GetCategories(bool aShowWaitSign)
	{
	QStringList params;
	params << "wp.getCategories" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetCategories), body.toUtf8()), aShowWaitSign, false));
	if (aShowWaitSign)
		emit TaskStarted(EGetCategories);
	}

void WPNetworkEngine::GetTags()
	{
	QStringList params;
	params << "wp.getTags" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetTags), body.toUtf8()), false, false));
	}

void WPNetworkEngine::GetPageStatuses()
	{
	QStringList params;
	params << "wp.getPageStatusList" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetPageStatuses), body.toUtf8()), false, false));
	}

void WPNetworkEngine::GetPostStatuses()
	{
	QStringList params;
	params << "wp.getPostStatusList" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetPostStatuses), body.toUtf8()), false, false));
	}

void WPNetworkEngine::GetCommentStatuses()
	{
	QStringList params;
	params << "wp.getCommentStatusList" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetCommentStatuses), body.toUtf8()), false, false));
	}

void WPNetworkEngine::GetComments(bool aShowWaitSign, TGetCommentsStatus aStatus, const QString& aPostId, int aNumberOfComments, int aOffset)
	{
	QMap<QString, QString> map;
	map.clear();
	if (aStatus  == EPendingComments)
		map.insert("status", "hold");
	if (aPostId  != "")
		map.insert("post_id", aPostId);
	if (aNumberOfComments != 10)
		map.insert("number", QString::number(aNumberOfComments));
	if (aOffset != 0)
		map.insert("offset", QString::number(aOffset));
	
	QStringList params;
	params << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body;
	CreateComplexRequest(body, "wp.getComments", params, &map, NULL, NULL);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EGetComments), body.toUtf8()), aShowWaitSign, true));
	if (aShowWaitSign)
		emit TaskStarted(EGetComments);
	}

void WPNetworkEngine::GetStats(const QString& postId) {
    if (iBlogData->iApiKey == "") {
        //iBlogData->iApiKey = "34694ef87bd0";
        return;
    }

    QUrl url = QUrl("http://stats.wordpress.com/csv.php");
    url.addQueryItem("api_key", iBlogData->iApiKey);
    url.addQueryItem("blog_id", iBlogData->iBlogId);
    url.addQueryItem("format", "xml");
    if (postId != "") {
        url.addQueryItem("table", "postviews");
        url.addQueryItem("post_id", postId);
    }
    iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->get(CreateNetworkRequest(url, EGetStats)), false, false));
}

void WPNetworkEngine::DeleteComment(const QString& aCommentId)
	{
	// Does not delete the actual comment from the local data
	QStringList params;
	params << "wp.deleteComment" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword << aCommentId;
	
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EDeleteComment), body.toUtf8()), true, true));
	emit TaskStarted(EDeleteComment);
	}

void WPNetworkEngine::HandleNetworkData(QNetworkReply *aNetworkReply)
{
	WPLogger::Instance().LogRow("NetworkEngine::HandleNetworkData called");
	
	WPNetworkQueueItem item;
	const int requestCount = iCurrentRequests.size();
	for (int i = 0; i < requestCount; i++)
	{
		if (iCurrentRequests[i].iNetworkReply == aNetworkReply)
		{
			item = iCurrentRequests[i];
			iCurrentRequests.removeAt(i);
			break;
		}
	}
    
	QVariant typeVariant = aNetworkReply->request().attribute(QNetworkRequest::Attribute(QNetworkRequest::User + 1));
    bool success;
    const int value = typeVariant.toInt(&success);
    if (!success)
	{
	    aNetworkReply->deleteLater();
		return; // TODO: Error handling?
	}
    
    if (item.iShowWaitNote)
    	emit TaskFinished(TWPNetworkEngineCommand(value));
	
    if (!aNetworkReply->error())
    	{
    	QByteArray rawResponse = aNetworkReply->readAll();
        QString response = QString::fromUtf8(rawResponse.constData(), rawResponse.length());
        
        switch (value)
        	{
        	case EGetMainPage:
        		{
        		QString rsdUrl;
        		int rsdIndex = response.indexOf("rel=\"EditURI\"");
        		if (rsdIndex > 0)
        			{
        			// Assuming that the HTML is valid at this point. If full certainty is wanted,
        			// we should make a new function that parses the string and has checks for all possible
        			// errors.
        			rsdIndex = response.indexOf("href", rsdIndex);
        			const int start = response.indexOf('\"', rsdIndex);
        			const int end = response.indexOf('\"', start + 1);
        			rsdUrl = response.mid(start + 1, end - start - 1); // stripping the '\"' characters
        			iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->get(CreateNetworkRequest(QUrl::fromUserInput(rsdUrl), EGetRSD)), true, true));
        			emit TaskStarted(EGetRSD);
        			}
        		else if (aNetworkReply->hasRawHeader("X-Pingback")) // If no rsd url found, using http header
        			{
        			GetUsersBlogs(QString::fromAscii(aNetworkReply->rawHeader("X-Pingback")));
        			}
        		else
        		{
        			QUrl possibleRedirectUrl(aNetworkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl());
        			if (!possibleRedirectUrl.isEmpty() && possibleRedirectUrl != aNetworkReply->url())
        			{
        				iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->get(CreateNetworkRequest(possibleRedirectUrl, EGetMainPage)), true, true));
        				emit TaskStarted(EGetMainPage);
        			}
            		// TODO: Error handling!
        		}
        		}
        		break;
        	case EGetRSD:
        		{
        		QList<WPDataRsdApi> rsdEndpoints;
        		WPXmlRsdParser parser(response, rsdEndpoints);
        		if (rsdEndpoints.size() > 0)
        			{
        			GetUsersBlogs(QUrl::fromUserInput(rsdEndpoints[0].iApiLink));
        			}
        		}
        		break;
        	case EGetUsersBlogs:
        		{
        		const int oldBlogs = iDataUsersBlogs->iBlogs.size();
        		WPXmlUsersBlogsParser parser(response, *iDataUsersBlogs);
                parser.ParseResponse();
                const int faultCode = parser.FaultCode();
                if (!faultCode)
                	{
                    // Basic filtering to drop duplicates out of the blog list
                    for (int i = oldBlogs; i < iDataUsersBlogs->iBlogs.size(); ++i)
                    {
                        for (int j = 0; j < oldBlogs; ++j)
                        {
                            const WPDataBlog& newBlog = iDataUsersBlogs->iBlogs[i];
                            const WPDataBlog& oldBlog = iDataUsersBlogs->iBlogs[j];
                            if (newBlog.iRpcUrl == oldBlog.iRpcUrl
                                && newBlog.iBlogId == oldBlog.iBlogId
                                && oldBlog.iUsername == iUsername)
                            {
                                iDataUsersBlogs->iBlogs.removeAt(i);
                                --i;
                                break;
                            }
                        }
                    }

                    // Updating the settings to the new blogs
                    const int newCount = iDataUsersBlogs->iBlogs.size();
                	for (int i = oldBlogs; i < newCount; i++)
                		{
						iDataUsersBlogs->iBlogs[i].iUsername = iUsername;
						iDataUsersBlogs->iBlogs[i].iPassword = iPassword;
						iDataUsersBlogs->iBlogs[i].iNumberOfPosts = iNumberOfPosts;
						iDataUsersBlogs->iBlogs[i].iResizePhotos = iResizePhotos;
                		}
                	if (newCount > oldBlogs)
                		{
                		iDataUsersBlogs->iActiveBlogIndex = oldBlogs;
                		SetCurrentBlog(iDataUsersBlogs->iBlogs[oldBlogs]);
                		}

            		emit UpdateData(*iDataUsersBlogs);
                	emit GetUserBlogsCompleted(*iBlogData);
                	RefreshAll(true); // Get all data for the current blog
                	}
                else
                {
                	switch (faultCode)
                	{
                		case 403:
                			emit ShowErrorNote(tr("Username and/or password do not match."));
                			break;
                		case 405:
                			emit ShowErrorNote(tr("XML RPC interface not enabled. Please enable it from your blog's admin interface and retry."));
                			break;
                		default:
                			emit ShowErrorNote(tr("Adding a new blog failed. Please check the blog settings."));
                			break;
                	}
                }
        		}
        		break;

        	// Refreshing data from network
        	case EGetComments:
        		{
        		WPLogger::Instance().LogRow("EGetComments parsing start");
        		iBlogData->iComments.clear();
        		WPXmlCommentsParser parser(response, *iBlogData);
                parser.ParseResponse();
        		WPLogger::Instance().LogRow("EGetComments parsing end");
                emit UpdateData(*iBlogData);
                emit UpdateCommentsData(*iBlogData);
        		WPLogger::Instance().LogRow("EGetComments signals sent");
        		}
        		break;
        	case EGetPosts:
        		{
        		iBlogData->iPosts.clear();
        		WPXmlPostsParser parser(response, *iBlogData);
                parser.ParseResponse();
/*
                QList<WPDataPost>::const_iterator it = iBlogData->iPosts.begin();
                while (it != iBlogData->iPosts.end()) {
                    GetStats((*(it++)).iPostId);
                }
*/
                emit UpdateData(*iBlogData);
                emit UpdatePostData(*iBlogData);
        		}
        		break;
        	case EGetPages:
        		{
        		iBlogData->iPages.clear();
        		WPXmlPagesParser parser(response, *iBlogData);
                parser.ParseResponse();
                emit UpdatePageData(*iBlogData);
        		}
        		break;
        	case EGetCategories:
        		{
        		iBlogData->iCategories.clear();
        		WPXmlCategoriesParser parser(response, *iBlogData);
                parser.ParseResponse();
                emit UpdateCategoryData(*iBlogData);
        		}
        		break;
        	case EGetTags:
        		{
        		iBlogData->iTags.clear();
        		WPXmlTagsParser parser(response, *iBlogData);
                parser.ParseResponse();
        		}
        		break;
        	case EGetPageStatuses:
        		{
        		iBlogData->iPageStatuses.clear();
        		WPXmlStatusesParser parser(response, iBlogData->iPageStatuses);
                parser.ParseResponse();
        		}
        		break;
        	case EGetPostStatuses:
        		{
        		iBlogData->iPostStatuses.clear();
        		WPXmlStatusesParser parser(response, iBlogData->iPostStatuses);
                parser.ParseResponse();
        		}
        		break;
        	case EGetCommentStatuses:
        		{
        		iBlogData->iCommentStatuses.clear();
        		WPXmlStatusesParser parser(response, iBlogData->iCommentStatuses);
                parser.ParseResponse();
        		}
        		break;

                case EGetStats:
                        {
                            WPXmlStatsParser parser(response, iBlogData->iStats);
                            parser.ParseResponse();
                        }
                        break;

        	// Updating data to network 
        	case ENewPage:
        		{
        		WPXmlResponseParser parser;
        		const bool success = parser.ParseResponse(response);
        		if (success && !iLocalDraftId.isEmpty())
        			iBlogData->RemoveLocalDraftPage(iLocalDraftId);
        		GetPages();
        		}
        		break;        	
        	case EEditPage:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
        		GetPages();
        		}
        		break;
        	case EDeletePage:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
        		GetPages();
        		}
        		break;
        	case ENewComment:
				{
				WPXmlResponseParser parser;
				bool success = parser.ParseResponse(response);
				GetComments();
				}
				break;
        	case EEditComment:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
        		GetComments();
        		}
        		break;
        	case EDeleteComment:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
        		GetComments();
        		}
        		break;
        	case ENewPost:
        		{
        		WPXmlResponseParser parser;
        		const bool success = parser.ParseResponse(response);
        		if (success && !iLocalDraftId.isEmpty())
        			iBlogData->RemoveLocalDraftPost(iLocalDraftId);
    			GetPosts();
        		}
        		break;
        	case EEditPost:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
    			GetPosts();
        		}
        		break;
        	case EDeletePost:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
    			GetPosts();
        		}
        		break;
        	case EGetGravatar:
        		{
        		QByteArray filename = ParseFilenameFromGravatarRequest(aNetworkReply->url());
        		QByteArray email = iGravatarHashes.value(filename);
        		iGravatarHashes.remove(filename);
#ifdef Q_OS_SYMBIAN
				QFile file(filename);
#else
				QString path(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
				path.append(".wordpress/");
				path.append(filename);
				QFile file(path);
#endif
        		if (!file.open(QIODevice::WriteOnly))
        		{
        		    aNetworkReply->deleteLater();
        			return; // TODO: Error handling?
        		}
        		const bool success = file.write(rawResponse) != -1;
        		file.close();
        		if (success)
        			emit GravatarDownloaded(email, filename);
        		}
        		break;
        	case ENewCategory:
        		{
        		WPXmlResponseParser parser;
        		bool success = parser.ParseResponse(response);
        		GetCategories();
        		}
        		break;
        	case EUploadFile:
        		{
        		QString url;
        		WPXmlFileUploadParser parser(response, url);
        		parser.ParseResponse(); // TODO: Error handling
				emit TaskFinished(EUploadFile);
        		QString& content = iPostContent["description"];
				content += " <p><a href=\"";
				content += url;
				content += "\"><img class=\"alignnone size-full wp-image-6\" src=\"";
				content += url;
				content += "\" /></a></p>";
				
				iFilenameIndex++;
				if (iFilenameIndex >= iFilenames.size())
					{
					// No more images to send
					iFilenames.clear();
					iFilenameIndex = 0;
					if (iUploadPost)
						{
						if (iPostId.isEmpty())
							NewPost(iPostContent, iPostListContent, iPublishPost, iFilenames, iLocalDraftId);
						else
							EditPost(iPostId, iPostContent, iPostListContent, iPublishPost, iFilenames);
						}
					else // Uploading page
						{
						if (iPostId.isEmpty())
							NewPage(iPostContent, iPublishPost, iFilenames, iLocalDraftId);
						else
							EditPage(iPostId, iPostContent, iPublishPost, iFilenames);
						}
					break;
					}
				
				// Otherwise send next image.
				UploadFile(iFilenames[iFilenameIndex]);
        		}
        		break;

        	default:
        		break;
        	}
    	}
    else
    {
		if (value == EGetGravatar && aNetworkReply->error() == QNetworkReply::ContentNotFoundError)
		{
			QByteArray filename = ParseFilenameFromGravatarRequest(aNetworkReply->url());
			QByteArray email = iGravatarHashes.value(filename);
			iGravatarHashes.remove(filename);
			iGravatar404Emails.insert(email);
			// We do not have to notify observers as default image is used by default
		}

		else if (value == ENewPost || value == ENewPage || (value == EUploadFile && iPostId.isEmpty()))
			SaveFailedToLocalDrafts();

		if (item.iShowErrorNote)
		{
			switch (aNetworkReply->error())
				{
				case QNetworkReply::TimeoutError:
					emit ShowErrorNote(tr("Connection timed out."));
					break;
				case QNetworkReply::ConnectionRefusedError:
					emit ShowErrorNote(tr("Connection refused."));
					break;
				case QNetworkReply::ContentNotFoundError:
					emit ShowErrorNote(tr("Content not found."));
					break;
				case QNetworkReply::RemoteHostClosedError:
					emit ShowErrorNote(tr("Server closed the connection."));
					break;
				case QNetworkReply::SslHandshakeFailedError:
					emit ShowErrorNote(tr("Secure connection could not be established."));
					break;
				case QNetworkReply::ContentAccessDenied:
					emit ShowErrorNote(tr("Access denied."));
					break;
				case QNetworkReply::HostNotFoundError:
					emit ShowErrorNote(tr("Host not found. Please check the blog URL."));
					break;
				case QNetworkReply::ProxyConnectionClosedError:
				case QNetworkReply::ProxyConnectionRefusedError:
				case QNetworkReply::ProxyNotFoundError:
				case QNetworkReply::ProxyTimeoutError:
				case QNetworkReply::ProxyAuthenticationRequiredError:
				case QNetworkReply::UnknownProxyError:
					emit ShowErrorNote(tr("Proxy server returned an error."));
					break;
				case QNetworkReply::AuthenticationRequiredError:
					emit ShowErrorNote(tr("Authentication not accepted."));
					break;
				case QNetworkReply::ContentReSendError:
				case QNetworkReply::ContentOperationNotPermittedError:
				case QNetworkReply::ProtocolUnknownError:
				case QNetworkReply::ProtocolInvalidOperationError:
				case QNetworkReply::UnknownNetworkError:
				case QNetworkReply::UnknownContentError:
				case QNetworkReply::ProtocolFailure:
					emit ShowErrorNote(tr("Connection error."));
					break;
				default:
					break;
				}
		}
    }
    aNetworkReply->deleteLater();
}

void WPNetworkEngine::SetCurrentBlog(WPDataBlog& aBlog)
{
	iBlogData = &aBlog;
}

void WPNetworkEngine::NewPage(const QMap<QString, QString>& aContent, const QString& aPublish,
		const QStringList& aFilenames, const QString& aLocalDraftId)
{
	iUploadPost = false;
	iLocalDraftId = aLocalDraftId;
	iFilenameIndex = 0;
	iPostId.clear();
	iFilenames = aFilenames;
	iPostContent = aContent;
	iPostListContent.clear();
	iPublishPost = aPublish;

	if (aFilenames.size() == 0)
	{
		QString body;
		QStringList paramList;
		paramList << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
		CreateComplexRequest(body, "wp.newPage", paramList, &aContent, NULL, &aPublish);
			
		iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, ENewPage), body.toUtf8()), true, true));
		emit TaskStarted(ENewPage);
	}
	else
	{
		// Uploading files first
		UploadFile(iFilenames[0]);
	}
}

void WPNetworkEngine::EditPage(const QString& aPageId, const QMap<QString, QString>& aContent, const QString& aPublish, const QStringList& aFilenames)
{
	iUploadPost = false;
	if (aFilenames.size() == 0)
	{
		QString body;
		QStringList paramList;
		paramList << iBlogData->iBlogId << aPageId << iBlogData->iUsername << iBlogData->iPassword;
		CreateComplexRequest(body, "wp.editPage", paramList, &aContent, NULL, &aPublish);
			
		iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EEditPage), body.toUtf8()), true, true));
		emit TaskStarted(EEditPage);
	}
	else
	{
		iFilenameIndex = 0;
		iPostId = aPageId;
		iFilenames = aFilenames;
		iPostContent = aContent;
		iPublishPost = aPublish;
		UploadFile(iFilenames[0]);
	}
}

void WPNetworkEngine::DeletePage(const QString& aPageId)
{
	QStringList params;
	params << "wp.deletePage" << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword << aPageId;
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EDeletePage), body.toUtf8()), true, true));
	emit TaskStarted(EDeletePage);
}

void WPNetworkEngine::NewPost(const QMap<QString, QString>& aContent,
		const QMap<QString, QStringList>& aListContent, const QString& aPublish,
		const QStringList& aFilenames, const QString& aLocalDraftId)
{
	iUploadPost = true;
	iLocalDraftId = aLocalDraftId;
	iFilenameIndex = 0;
	iPostId.clear();
	iFilenames = aFilenames;
	iPostContent = aContent;
	iPostListContent = aListContent;
	iPublishPost = aPublish;

	if (aFilenames.size() == 0)
	{
		QString body;
		QStringList paramList;
		paramList << iBlogData->iBlogId
				  << iBlogData->iUsername
				  << iBlogData->iPassword;
		CreateComplexRequest(body, "metaWeblog.newPost", paramList, &aContent, &aListContent, &aPublish);
		
		iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, ENewPost), body.toUtf8()), true, true));
		emit TaskStarted(ENewPost);
	}
	else
	{
		// Otherwise, we will first upload the files.
		UploadFile(iFilenames[0]);
	}
}

void WPNetworkEngine::EditPost(const QString& aPostId, const QMap<QString, QString>& aContent,
		const QMap<QString, QStringList>& aListContent, const QString& aPublish, const QStringList& aFilenames)
{
	iUploadPost = true;
	if (aFilenames.size() == 0)
	{
		QString body;
		QStringList paramList;
		paramList << aPostId 
				  << iBlogData->iUsername
				  << iBlogData->iPassword;
		CreateComplexRequest(body, "metaWeblog.editPost", paramList, &aContent, &aListContent, &aPublish);
		
		iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EEditPost), body.toUtf8()), true, true));
		emit TaskStarted(EEditPost);
	}
	else
	{
		// Let's first upload the files
		iFilenameIndex = 0;
		iPostId = aPostId;
		iFilenames = aFilenames;
		iPostContent = aContent;
		iPostListContent = aListContent;
		iPublishPost = aPublish;
		UploadFile(iFilenames[0]);
	}
}

void WPNetworkEngine::DeletePost(const QString& aPostId, const QString& aPublish)
{
	QStringList params;
	// TODO Using "123" as an app key. Should something else be used? http://www.xmlrpc.com/metaWeblogApi
	params << "metaWeblog.deletePost" << "123" << aPostId << iBlogData->iUsername << iBlogData->iPassword << aPublish;
	QString body = CreateSimpleRequest(params);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EDeletePost), body.toUtf8()), true, true));
	emit TaskStarted(EDeletePost);
}

void WPNetworkEngine::NewComment(const QString& aComment, const QString& aPostId, const QString& aCommentParent)
{
	QString body;
	QStringList paramList;
	paramList << iBlogData->iBlogId
	          << iBlogData->iUsername
	          << iBlogData->iPassword
	          << aPostId;
	QMap<QString, QString> map;
	map.insert("comment_parent", aCommentParent);
	map.insert("content", aComment);
	CreateComplexRequest(body, "wp.newComment", paramList, &map, NULL, NULL);
	
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, ENewComment), body.toUtf8()), true, true));
	emit TaskStarted(ENewComment);
}

void WPNetworkEngine::EditComment(const QString& aCommentId, const QString& aStatus)
	{
	QString body;
	QStringList paramList;
	paramList << iBlogData->iBlogId
	          << iBlogData->iUsername
	          << iBlogData->iPassword
	          << aCommentId;
	QMap<QString, QString> map;
	map.insert("status", aStatus);
	CreateComplexRequest(body, "wp.editComment", paramList, &map, NULL, NULL);
	
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EEditComment), body.toUtf8()), true, true));
	emit TaskStarted(EEditComment);
	}

QString WPNetworkEngine::CreateSimpleRequest(const QStringList& aRequestParams)
{
	QString body;
	QXmlStreamWriter writer(&body);
	writer.writeStartDocument("1.0");
	writer.writeStartElement("methodCall");
	writer.writeTextElement("methodName", aRequestParams.at(0));
	writer.writeStartElement("params");
	
	for (int i = 1; i < aRequestParams.size(); i++)
	{
		writer.writeStartElement("param");
		writer.writeStartElement("value");
		writer.writeTextElement("string", aRequestParams.at(i));
		writer.writeEndElement(); // value
		writer.writeEndElement(); // param
	}
	
	writer.writeEndElement(); // params
	writer.writeEndElement(); // methodCall
	

	return body;
}

void WPNetworkEngine::CreateComplexRequest(QString& aRequest, const QString& aMethodName,
		const QStringList& aParamList, const QMap<QString, QString>* aContent,
		const QMap<QString, QStringList>* aListContent, const QString* aPublish)
	{
	QXmlStreamWriter writer(&aRequest);
	// UTF-8 is used by default
	writer.writeStartDocument("1.0");
	writer.writeStartElement("methodCall");
	writer.writeTextElement("methodName", aMethodName);
	writer.writeStartElement("params");
	
	foreach (QString value, aParamList)
	{
		writer.writeStartElement("param");
		writer.writeStartElement("value");
		writer.writeTextElement("string", value);
		writer.writeEndElement(); // value
		writer.writeEndElement(); // param
	}

	writer.writeStartElement("param");
	writer.writeStartElement("value");
	writer.writeStartElement("struct");
	QMap<QString, QString>::const_iterator i;
	for (i = aContent->constBegin(); i != aContent->constEnd(); ++i)
	{
		writer.writeStartElement("member");
		writer.writeTextElement("name", i.key());
		writer.writeStartElement("value");
		// This should be the only element that describes
		// dateTime values in outbound messages
		if (i.key() == "dateCreated")
			writer.writeTextElement("dateTime.iso8601", i.value());
		else
			writer.writeTextElement("string", i.value());
		writer.writeEndElement(); // value
		writer.writeEndElement(); // member
	}
	if (aListContent != NULL)
	{
		QMap<QString, QStringList>::const_iterator j;
		for (j = aListContent->constBegin(); j != aListContent->constEnd(); ++j)
		{
			writer.writeStartElement("member");
			writer.writeTextElement("name", j.key());
			writer.writeStartElement("value");
			writer.writeStartElement("array");
			writer.writeStartElement("data");
			foreach (QString str, j.value())
			{
				writer.writeStartElement("value");
				writer.writeTextElement("string", str);
				writer.writeEndElement(); // value
			}
			writer.writeEndElement(); // data
			writer.writeEndElement(); // array
			writer.writeEndElement(); // value
			writer.writeEndElement(); // member
		}
	}
	writer.writeEndElement(); // struct
	writer.writeEndElement(); // value
	writer.writeEndElement(); // param
	
	if (aPublish)
	{
		writer.writeStartElement("param");
		writer.writeStartElement("value");
		writer.writeTextElement("string", *aPublish);
		writer.writeEndElement(); // value
		writer.writeEndElement(); // param
	}
	
	writer.writeEndElement(); // params
	writer.writeEndElement(); // methodcall
	}

QNetworkRequest WPNetworkEngine::CreateNetworkRequest(const QUrl& aUrl, TWPNetworkEngineCommand aType)
	{
	QNetworkRequest request(aUrl);
	request.setAttribute(QNetworkRequest::Attribute(QNetworkRequest::User + 1), QVariant((int)aType));
	request.setRawHeader("User-Agent", "wp-nokia/0.5.4");
	return request;
	}

QByteArray WPNetworkEngine::MD5Hash(const QByteArray& aSource)
	{
	MD5Context context;
	MD5Init(&context);
	MD5Update(&context, reinterpret_cast<const unsigned char*>(aSource.constData()), aSource.length());
	unsigned char hashValue[16];
	MD5Final(&context, hashValue);
	QByteArray returnValue;
	for (int i = 0; i < 16; i++)
		{
		if (hashValue[i] < 16)
			returnValue += "0";
		returnValue += QByteArray::number(hashValue[i], 16);
		}
	return returnValue;
	}

void WPNetworkEngine::GetGravatar(const QString& aEmail)
	{
	QByteArray email8(aEmail.toAscii());
	QByteArray hash(MD5Hash(email8));

	hash += ".jpg";
        QString fileName;
#ifdef Q_OS_SYMBIAN
        fileName = hash;
#else
        fileName = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
        fileName += ".wordpress/";
        fileName += hash;
#endif
        if (QFile::exists(fileName))
		{
			emit GravatarDownloaded(aEmail, fileName); // TODO: Cache invalidation at some point?
			return;
		}
	
	if (iGravatarHashes.contains(hash))
		{
		emit GravatarDownloaded(aEmail, KDefaultAvatar); // Default image while downloading
		return; // Email is being asked from the network, so let's wait for the response
		}
	
	if (iGravatar404Emails.contains(email8))
		{
		emit GravatarDownloaded(aEmail, KDefaultAvatar);
		return;
		}
	
	iGravatarHashes.insert(hash, email8);
	QString url = "http://www.gravatar.com/avatar/";
	url += hash;
	url += "?d=404";
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->get(CreateNetworkRequest(url, EGetGravatar)), false, false));
	emit GravatarDownloaded(aEmail, KDefaultAvatar); // Let's use the default icon if no gravatar found
	}

void WPNetworkEngine::NewCategory(const QString& aName, const QString& aParentId)
	{
	QMap<QString, QString> map;
	map.clear();
	map.insert("name", aName);
	if (aParentId  != "")
		map.insert("parent_id", aParentId);
	
	QStringList params;
	params << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QString body;
	CreateComplexRequest(body, "wp.newCategory", params, &map, NULL, NULL);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, ENewCategory), body.toUtf8()), true, true));
	emit TaskStarted(ENewCategory);
	}

QByteArray WPNetworkEngine::ParseFilenameFromGravatarRequest(const QUrl& aUrl)
	{
	QString url = aUrl.toString();
	const int index = url.lastIndexOf('/');
	const int end = url.lastIndexOf('?');
	return url.mid(index + 1, end - index - 1).toAscii();
	}

QString WPNetworkEngine::LocalizedTime(const QDateTime& aDateTime)
{
	QLocale locale;
	QString localStringFormat = locale.dateTimeFormat(QLocale::ShortFormat);
	return aDateTime.toString(localStringFormat);
}

QString WPNetworkEngine::LocalizedTime(const QString& aTimeString)
{
	return LocalizedTime(QDateTime::fromString(aTimeString, "yyyyMMddTHH:mm:ss"));
}

QString WPNetworkEngine::ShortLocalizedTime(const QString& aTimeString)
{
	QDateTime time(QDateTime::fromString(aTimeString, "yyyyMMddTHH:mm:ss"));
	return time.toString("MM/dd HH:mm");
}

QString WPNetworkEngine::ISO8601Time(const QDateTime& aDateTime)
{
	return aDateTime.toString("yyyyMMddTHH:mm:ss");
}

void WPNetworkEngine::ClearRequestQueue()
	{
	// We copy the request list and call abort to each request on the copied array.
	// This assures that all requests will be aborted, no matter whether 
	// abort() function is synchronous or asynchronous.
	QList<WPNetworkQueueItem> requests;
	const int requestCount = iCurrentRequests.size();
	for (int i = 0; i < requestCount; i++)
		requests.append(iCurrentRequests[i]);
	for (int i = 0; i < requestCount; i++)
		requests[i].iNetworkReply->abort();
	}

bool WPNetworkEngine::DeleteBlog(const QString& aBlogId)
	{
	const int numBlogs = iDataUsersBlogs->iBlogs.size();
	if (numBlogs < 2)
		return false; // Impossible to delete the last blog

	WPDataBlog* deleted = NULL;
	int index = -1;
	for (int i = 0; i < numBlogs; ++i)
	{
		WPDataBlog& blog = iDataUsersBlogs->iBlogs[i];
		QString id = blog.iRpcUrl;
		id += blog.iBlogId;
		if (id == aBlogId)
			{
			deleted = &blog;
			index = i;
			break;
			}
	}
	if (!deleted)
		return false; // Wrong id, should not happen but a sanity check
	if (deleted != iBlogData)
		{
		iDataUsersBlogs->iBlogs.removeAt(index);
		return true;
		}
	
	// The user wants to delete the current blog. So let's activate the next blog.
	const int oldIndex = index;
	index++;
	if (index == numBlogs)
		index = 0;

	WPDataBlog& blog = iDataUsersBlogs->iBlogs[index];
	OpenBlog(blog.iRpcUrl + blog.iBlogId);
	iDataUsersBlogs->iBlogs.removeAt(oldIndex);
	return true;
	}

void WPNetworkEngine::UploadFile(const QString& aFilename)
	{
	QStringList params;
	params << iBlogData->iBlogId << iBlogData->iUsername << iBlogData->iPassword;
	
	QByteArray body;
	CreateUploadFileRequest(body, params, aFilename);
	iCurrentRequests.append(WPNetworkQueueItem(iNetworkAccessManager->post(CreateNetworkRequest(iBlogData->iRpcUrl, EUploadFile), body), true, true));
	emit TaskStarted(EUploadFile);
	}

void WPNetworkEngine::CreateUploadFileRequest(QByteArray& aRequest, const QStringList& aParamList,
    const QString& aFilename) const
	{
	QFile file(aFilename);
    aRequest.clear();
	if (!file.open(QIODevice::ReadOnly))
		{
		return;
		}
	
	QXmlStreamWriter writer(&aRequest);
	// UTF-8 is used by default
	writer.writeStartDocument("1.0");
	writer.writeStartElement("methodCall");
	writer.writeTextElement("methodName", "wp.uploadFile");
	writer.writeStartElement("params");
	
	foreach (QString value, aParamList)
	{
		writer.writeStartElement("param");
		writer.writeStartElement("value");
		writer.writeTextElement("string", value);
		writer.writeEndElement(); // value
		writer.writeEndElement(); // param
	}

	writer.writeStartElement("param");
	writer.writeStartElement("value");
	writer.writeStartElement("struct");
	writer.writeStartElement("member");
	writer.writeTextElement("name", "name");
	writer.writeStartElement("value");
	
	const int pathend = aFilename.lastIndexOf('/');
	writer.writeTextElement("string", aFilename.mid(pathend + 1));
	writer.writeEndElement(); // value
	writer.writeEndElement(); // member

	writer.writeStartElement("member");
	writer.writeTextElement("name", "bits");
	writer.writeStartElement("value");
	if (iBlogData->iResizePhotos)
		{
		file.close();
		QImageReader reader;
		reader.setFileName(aFilename);
		QSize imageSize = reader.size();
		if (qMax(imageSize.height(), imageSize.width()) > 640)
			{
			imageSize.scale(640, 640, Qt::KeepAspectRatio);
			reader.setScaledSize(imageSize);
			QImage image = reader.read();
			if (!image.isNull())
				{
				QByteArray ba;
				QBuffer buffer(&ba);
				buffer.open(QIODevice::WriteOnly);
				image.save(&buffer, "JPG");
				aRequest.reserve(aRequest.length() + ba.length() * 4 / 3 + 128);
				ba = ba.toBase64(); // Frees the memory of the original image data
				QString string(ba);
				ba.clear(); // Frees the memory of base64 encoded image data
				writer.writeTextElement("base64", string);
				}
			else
				{
		        aRequest.clear();
				return;
				}
			}
		else // No need to scale the image
			{
			// TODO: Figure out a cleaner solution.
			QIODevice* device = reader.device();
			if (!device || !device->isOpen())
				{
				if (!file.open(QIODevice::ReadOnly))
					{
			        aRequest.clear();
					return;
					}
				aRequest.reserve(aRequest.length() + file.bytesAvailable() * 4 / 3 + 128);
				QByteArray ba(file.readAll());
				ba = ba.toBase64(); // Frees the memory of the original image data
				QString string(ba);
				ba.clear(); // Frees the memory of 8-bit data
				writer.writeTextElement("base64", string);
				file.close();
				}
			else
				{
				aRequest.reserve(aRequest.length() + device->bytesAvailable() * 4 / 3 + 128);
				QByteArray ba(device->readAll());
				ba = ba.toBase64();
				QString string(ba);
				ba.clear();
				writer.writeTextElement("base64", string);
				}
			}
		}
	else
		{
	    writer.writeStartElement("base64");
	    
	    // A chunked implementation of the base64 encoding. It should work
	    // if the chunk size is a multiple of 3. This makes sure that we have a correct
	    // UTF-8 encoding and we do not have to create huge temporary buffers.
	    
	    const int KChunkSize = 3 * 32 * 1024; // 96 kb chunks
	    qint64 bytesAvailable = file.bytesAvailable();
	    
	    // Reserving a big enough buffer. The last constant includes some extra to really
	    // avoid overallocation.
        aRequest.reserve(aRequest.length() + bytesAvailable * 4 / 3 + 128);
	    while (bytesAvailable > 0)
	    {
			QByteArray ba(file.read(KChunkSize));
			ba = ba.toBase64();
			QString string(ba);
			ba.clear();
	        writer.writeCharacters(string);
	        bytesAvailable = file.bytesAvailable();
	    }
		writer.writeEndElement(); // base64
		file.close();
		}

	writer.writeEndElement(); // value
	writer.writeEndElement(); // member

	writer.writeEndElement(); // struct
	writer.writeEndElement(); // value
	writer.writeEndElement(); // param
	
	writer.writeEndElement(); // params
	writer.writeEndElement(); // methodcall
	}

void WPNetworkEngine::SaveFailedToLocalDrafts()
{
	if (iUploadPost && iLocalDraftId.isEmpty())
	{
		WPDataPost post;
		QString id = "local_";
		id += QString::number(qrand());
		post.iPostId = id;
		post.iTitle = iPostContent["title"];
		post.iCategories = iPostListContent["categories"];
		post.iDescription = iPostContent["description"];
		post.iMtKeywords = iPostContent["mt_keywords"];
		post.iDateCreated = iPostContent["dateCreated"];
		post.iWpPassword = iPostContent["wp_password"];
		iBlogData->iLocalDraftPosts.append(post);
		emit UpdatePostData(*iBlogData);
		return;
	}
	if (iUploadPost) // Updating old local draft when publishing failed
	{
		WPDataPost* localDraft = GetLocalDraftPost(iLocalDraftId);
		if (!localDraft)
			return;
		localDraft->iTitle = iPostContent["title"];
		localDraft->iCategories = iPostListContent["categories"];
		localDraft->iDescription = iPostContent["description"];
		localDraft->iMtKeywords = iPostContent["mt_keywords"];
		localDraft->iDateCreated = iPostContent["dateCreated"];
		localDraft->iWpPassword = iPostContent["wp_password"];
		emit UpdatePostData(*iBlogData);
		return;
	}
	if (iLocalDraftId.isEmpty())
	{
		WPDataPage page;
		QString id = "local_";
		id += QString::number(qrand());
		page.iPageId = id;
		page.iTitle = iPostContent["title"];
		page.iDescription = iPostContent["description"];
		page.iDateCreated = iPostContent["dateCreated"];
		iBlogData->iLocalDraftPages.append(page);
		emit UpdatePageData(*iBlogData);
		return;
	}
	WPDataPage* localDraft = GetLocalDraftPage(iLocalDraftId);
	if (!localDraft)
		return;
	localDraft->iTitle = iPostContent["title"];
	localDraft->iDescription = iPostContent["description"];
	localDraft->iDateCreated = iPostContent["dateCreated"];
	emit UpdatePageData(*iBlogData);
}
