
#include <QVBoxLayout>
#include "wpcommentlistitem.h"
#include "wpcommentlist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "commentsview.h"
#include "wpscrollarea.h"

#include "PopupMenu.h"
#include "WPLogger.h"

#ifdef Q_WS_MAEMO_5
	static const int KTitleDefaultHeight = 45;
#else
	static const int KTitleDefaultHeight = 32;
#endif

WPCommentList::WPCommentList(QWidget* aParent, const QList<WPDataComment>& aData,
		WPNetworkEngine& aNetworkEngine, WPScrollArea& aScrollArea, bool aShowFullComment)
: MDragWidget(aParent), iData(aData), iNetworkEngine(aNetworkEngine), iScrollArea(aScrollArea)
{
	iLayout = new QVBoxLayout;
	iLayout->setSpacing(0);
	iLayout->setContentsMargins(0,0,0,0);
	
	iActionApprove = new QAction(tr("Approve"), this);
	iActionUnapprove = new QAction(tr("Unapprove"), this);
	iActionView = new QAction(tr("View"), this);
	iActionHide = new QAction(tr("Minimize"), this);
	iActionReply = new QAction(tr("Reply"), this);
	iActionDelete = new QAction(tr("Delete"), this);
	iActionSpam = new QAction(tr("Spam"), this);
	iPopupMenu = NULL;
	
	connect(iActionApprove,   SIGNAL(triggered()), this, SLOT(Approve()));
	connect(iActionUnapprove, SIGNAL(triggered()), this, SLOT(Unapprove()));
	connect(iActionView,      SIGNAL(triggered()), this, SLOT(View()));
	connect(iActionHide,      SIGNAL(triggered()), this, SLOT(View()));
	connect(iActionReply,     SIGNAL(triggered()), this, SLOT(Reply()));
	connect(iActionDelete,    SIGNAL(triggered()), this, SLOT(Delete()));
	connect(iActionSpam,      SIGNAL(triggered()), this, SLOT(Spam()));
	
	connect(&iNetworkEngine, SIGNAL(GravatarDownloaded(const QString, const QString&)),
			this, SLOT(GetGravatarCompleted(const QString&, const QString&)));

	QList<const WPDataComment*> unapprovedList;
	QList<const WPDataComment*> approvedList;
	const int dataCount = aData.size();
	for (int i = 0; i < dataCount; i++)
	{
		if (aData[i].iStatus == "hold")
			unapprovedList.append(&(aData[i]));
		else
			approvedList.append(&(aData[i]));
	}
	
	// Unapproved comments
	const int unapprovedCount = unapprovedList.size();
	if (unapprovedCount > 0)
	{
		QLabel* unapprovedLabel = new QLabel(this);
		unapprovedLabel->setObjectName(QString::fromUtf8("unapprovedLabel"));
		QString unapproved = tr("Unapproved") + " (" + QString::number(unapprovedCount) + ")";
		unapprovedLabel->setFixedHeight(KTitleDefaultHeight);
		unapprovedLabel->setText(unapproved);
		iLayout->addWidget(unapprovedLabel);
		for (int i = 0; i < unapprovedCount; i++)
			AddComment(*unapprovedList[i], aShowFullComment);
	}

	// All other comments
	const int approvedCount = approvedList.size();
	if (approvedCount > 0)
	{
		QLabel* approvedLabel = new QLabel(this);
		approvedLabel->setObjectName(QString::fromUtf8("approvedLabel"));
		QString approved = tr("Comments") + " (" + QString::number(approvedCount) + ")";
		approvedLabel->setFixedHeight(KTitleDefaultHeight);
		approvedLabel->setText(approved);
		iLayout->addWidget(approvedLabel);
		for (int i = 0; i < approvedCount; i++)
			AddComment(*approvedList[i], aShowFullComment);
	}

	// Gravatars
	WPLogger::Instance().LogRow("Starting to get Gravatars.");
	
	// Small optimization to only ask gravatars once per email. This helps a lot if the
	// comment list has many comments from the same email address.
	QStringList askedList;
	for (int i = 0; i < dataCount; ++i)
	{
	    if (!askedList.contains(aData[i].iAuthorEmail, Qt::CaseInsensitive))
	    {
	        askedList.append(aData[i].iAuthorEmail);
	        iNetworkEngine.GetGravatar(aData[i].iAuthorEmail);
	    }
	}
	WPLogger::Instance().LogRow("Gravatars asked.");	
	
	// Sets a minimum height for this container. We can't remove items from this,
	// so we can determine the minimum size pretty accurately: item height plus
	// margin per item.
	// TODO not working any more; how to account for the two headers in the list?
	int height = 0;
	if (approvedCount > 0)
		height += KTitleDefaultHeight; // Height + margin
	if (unapprovedCount > 0)
		height += KTitleDefaultHeight; // Height + margin
	
	height += aData.count() * (WPCommentListItem::ItemMinimumHeight); // Height + margin
	setMinimumHeight(height);
	
	iLayout->addStretch();
	setLayout(iLayout);
}

WPCommentList::~WPCommentList()
{

}

void WPCommentList::AddComment(const WPDataComment& aComment, bool aShowFullComment)
{
	WPCommentListItem* item = new WPCommentListItem(*this, aComment, aShowFullComment);
	// Minimum size for the box -- this should some avoid unnecessary redraws
	item->setMinimumHeight(WPCommentListItem::ItemMinimumHeight);
	connect(item, SIGNAL(CommentItemSelected(WPCommentListItem*)), this, SLOT(ShowContextMenu(WPCommentListItem*)));
	iItems.append(item);
	iLayout->addWidget(item);
}

void WPCommentList::GetGravatarCompleted(const QString& aEmail, const QString& aFilename)
{
	const int count = iItems.count();
	for (int i = 0; i < count; i++)
	{
		if (iItems[i]->GetComment().iAuthorEmail == aEmail)
			iItems[i]->SetAvatarImage(aFilename); // No break as image can apply to multiple comments
	}
}

void WPCommentList::ShowContextMenu(WPCommentListItem* aItem)
{
	if (MDragWidgetIsDragging() || iPopupMenu)
		return;
	
	iPopupMenu = new PopupMenu(this);
	connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
	iScrollArea.DisableScrollAtWidget(iPopupMenu);
	iCurrentItem = aItem;
	if (aItem->GetComment().iStatus != "approve")
	{
	    iPopupMenu->addAction(iActionApprove);
	    iPopupMenu->setActiveAction(iActionApprove);
	    iPopupMenu->setFocus();
	}
	else
	{
	    iPopupMenu->addAction(iActionUnapprove);
	    iPopupMenu->setActiveAction(iActionUnapprove);
	    iPopupMenu->setFocus();
	}
	if (aItem->ShowFullComment())
	    iPopupMenu->addAction(iActionHide);
	else
	    iPopupMenu->addAction(iActionView);
	if (iNetworkEngine.GetPost(aItem->GetComment().iPostId))
	    iPopupMenu->addAction(iActionReply);
	iPopupMenu->addAction(iActionDelete);
	if (aItem->GetComment().iStatus != "spam")
	    iPopupMenu->addAction(iActionSpam);
	
	QPoint itemPoint = aItem->TopRight();
	itemPoint.setY(itemPoint.y() - 5);
	QPoint globalItemPoint = mapToGlobal(itemPoint);
	QSize size = iPopupMenu->sizeHint();

	// Set popup a couple of pixels away from the screen right edge
	itemPoint.setX(itemPoint.x() - size.width() + 20); 
	// Make sure the popup won't be placed over the tab bar
	if (globalItemPoint.y() < iScrollArea.TopRightInGlobal().y())
	{
		itemPoint.setY(itemPoint.y() - globalItemPoint.y() + iScrollArea.TopRightInGlobal().y());	
	}

	iPopupMenu->popup(mapToGlobal(itemPoint));
}

bool WPCommentList::MDragWidgetIsDragging() const
{
	return iScrollArea.IsDragging();
}

QSize WPCommentList::VisibleSize() const
{
	return iScrollArea.GetVisibleContentSize();
}

void WPCommentList::Approve()
{
	iNetworkEngine.EditComment(iCurrentItem->GetComment().iCommentId, "approve");
}

void WPCommentList::Unapprove()
{
	iNetworkEngine.EditComment(iCurrentItem->GetComment().iCommentId, "hold");
}

void WPCommentList::View()
{
	iCurrentItem->ChangeViewMode();
}

void WPCommentList::Reply()
{
	emit ReplyToComment(iCurrentItem->GetComment());
}

void WPCommentList::Delete()
{
	iNetworkEngine.DeleteComment(iCurrentItem->GetComment().iCommentId);
}

void WPCommentList::Spam()
{
	iNetworkEngine.EditComment(iCurrentItem->GetComment().iCommentId, "spam");
}

void WPCommentList::setFocus()
{
	if (iItems.count() > 0)
	{
		iItems.at(0)->setFocus();
	}
	else
	{
		QFrame::setFocus();
	}
}

void WPCommentList::MenuHiding()
{
    iScrollArea.RemoveSelfScrollingWidget(iPopupMenu);
    iPopupMenu->deleteLater();
    iPopupMenu = NULL;
}
